@@ -29,6 +29,12 @@
 // Everything in this module is implementation for RepoStreamer.
 // Extractor is also used outside of here, but only trivially.
 //
+// Nothing in here is dependent on the DSL surface syntax.  It does
+// assume logit(), announce(), croak(), throw() and catch() do sane things.
+// The code also assumes a control block that includes a baton for
+// reports.
+//
+// Copyright by Eric S. Raymond
 // SPDX-License-Identifier: BSD-2-Clause
 
 package main
@@ -93,10 +99,11 @@ type manifestEntry struct {
 type Extractor interface {
 	// Hook for any setup actions required before streaming
 	preExtract()
-	// Hook called periodically for various housekeeping actions
+	// Hook called periodically (every 1000 commits) for
+	// any housekeeping actions required.  Optional.
 	keepHouse() error
-	// Gather the topologically-ordered lists of revisions and the parent map
-	// (revlist and parent members)
+	// Gather the topologically-ordered lists of revisions and the
+	// parent map (revlist and parent members)
 	gatherRevisionIDs(*RepoStreamer) error
 	// Gather all other per-commit data except branch IDs
 	// (ai and ci members in self.meta list elements)
@@ -161,7 +168,9 @@ func (cm *ColorMixer) simulateGitColoring(mc MixerCapable, base *RepoStreamer) {
 	cm.childStamps = mc.gatherChildTimestamps(base)
 	if logEnable(logTOPOLOGY) {
 		for _, rev := range cm.base.revlist {
-			logit(logSHOUT, "Revision %s has branch '%s'\n", rev, cm.base.meta[rev].branch)
+			if logEnable(logSHOUT) {
+				logit("Revision %s has branch '%s'\n", rev, cm.base.meta[rev].branch)
+			}
 		}
 	}
 	// Depends on the order of the revlist to be correct.
@@ -169,7 +178,9 @@ func (cm *ColorMixer) simulateGitColoring(mc MixerCapable, base *RepoStreamer) {
 	// not necessary id your VCS dumps branches in
 	// revlist-tip order.
 	for _, refname := range base.refs.keys {
-		logit(logTOPOLOGY, "outside branch coloring %s %s", base.refs.get(refname), refname)
+		if logEnable(logTOPOLOGY) {
+			logit("outside branch coloring %s %s", base.refs.get(refname), refname)
+		}
 		cm._branchColor(base.refs.get(refname), refname)
 	}
 }
@@ -185,7 +196,9 @@ func (cm *ColorMixer) _branchColor(rev, color string) {
 	if cm.base.branchesAreColored && strings.HasPrefix(color, "refs/heads/") {
 		return
 	}
-	logit(logTOPOLOGY, "inside branch coloring %s %s", rev, color)
+	if logEnable(logTOPOLOGY) {
+		logit("inside branch coloring %s %s", rev, color)
+	}
 	// This ensures that a branch tip rev never gets colored over
 	if _, ok := cm.childStamps[rev]; !ok {
 		cm.childStamps[rev] = farFuture
@@ -193,10 +206,14 @@ func (cm *ColorMixer) _branchColor(rev, color string) {
 	// This is used below to ensure that a branch color is never colored
 	// back to a tag
 	isBranchColor := strings.HasPrefix(color, "refs/heads/")
-	logit(logTOPOLOGY, "%s is-a-branch is %v", color, isBranchColor)
+	if logEnable(logTOPOLOGY) {
+		logit("%s is-a-branch is %v", color, isBranchColor)
+	}
 	unassigned := func(rev string) bool {
 		u := (cm.base.meta[rev].branch == "")
-		logit(logTOPOLOGY, "%s unassigned is %v", rev, u)
+		if logEnable(logTOPOLOGY) {
+			logit("%s unassigned is %v", rev, u)
+		}
 		return u
 	}
 	onTagBranch := func(rev string) bool {
@@ -218,13 +235,17 @@ func (cm *ColorMixer) _branchColor(rev, color string) {
 		// precedence over branches, so we never color back to a tag with
 		// a branch color
 		var parents []string
-		logit(logTOPOLOGY, "parents of %s (%s) before filtering %v", rev, timestamp.UTC(), cm.base.getParents(rev))
+		if logEnable(logTOPOLOGY) {
+			logit("parents of %s (%s) before filtering %v", rev, timestamp.UTC(), cm.base.getParents(rev))
+		}
 		for _, p := range cm.base.getParents(rev) {
 			if unassigned(p) || ((!(isBranchColor && onTagBranch(p))) && (cm.childStamps[p].Before(timestamp))) {
 				parents = append(parents, p)
 			}
 		}
-		logit(logTOPOLOGY, "parents of %s are %v", rev, parents)
+		if logEnable(logTOPOLOGY) {
+			logit("parents of %s are %v", rev, parents)
+		}
 
 		if len(parents) == 0 {
 			break
@@ -248,15 +269,6 @@ func (cm *ColorMixer) _branchColor(rev, color string) {
 	}
 }
 
-func findVCS(name string) *VCS {
-	for _, vcs := range vcstypes {
-		if vcs.name == name {
-			return &vcs
-		}
-	}
-	panic("reposurgeon: failed to find '" + name + "' in VCS types")
-}
-
 func lineByLine(rs *RepoStreamer, command string, errfmt string,
 	hook func(string, *RepoStreamer) error) error {
 	stdout, cmd, err1 := readFromProcess(command)
@@ -371,7 +383,7 @@ func (ge *GitExtractor) gatherAllReferences(rs *RepoStreamer) error {
 			return err2
 		}
 		tag := strings.Trim(fline, "\n")
-		taghash, terr := captureFromProcess("git rev-parse " + tag)
+		taghash, terr := captureFromProcess("git rev-parse "+tag, control.baton)
 		if terr != nil {
 			panic(throw("extractor", "Could not spawn git rev-parse: %v", terr))
 		}
@@ -441,7 +453,7 @@ func (ge *GitExtractor) colorBranches(rs *RepoStreamer) error {
 		return err1
 	}
 	defer markfile.Close()
-	data, err3 := captureFromProcess("git fast-export --all --export-marks=" + file.Name())
+	data, err3 := captureFromProcess("git fast-export --all --export-marks="+file.Name(), control.baton)
 	if err3 != nil {
 		panic(throw("extractor", "Couldn't spawn git-fast-export: %v", err3))
 	}
@@ -484,7 +496,7 @@ func (ge *GitExtractor) colorBranches(rs *RepoStreamer) error {
 }
 
 func (ge *GitExtractor) _metadata(rev string, format string) string {
-	line, err := captureFromProcess(fmt.Sprintf("git log -1 --format='%s' %s", format, rev))
+	line, err := captureFromProcess(fmt.Sprintf("git log -1 --format='%s' %s", format, rev), control.baton)
 	if err != nil {
 		panic(throw("extractor", "Couldn't spawn git log: %v", err))
 	}
@@ -504,7 +516,7 @@ func (ge *GitExtractor) postExtract(_repo *Repository) {
 
 // isClean is a predicate;  return true if repo has no unsaved changes.
 func (ge *GitExtractor) isClean() bool {
-	data, err := captureFromProcess("git ls-files --modified")
+	data, err := captureFromProcess("git ls-files --modified", control.baton)
 	if err != nil {
 		panic(throw("extractor", "Couldn't spawn git ls-files --modified: %v", err))
 	}
@@ -513,7 +525,7 @@ func (ge *GitExtractor) isClean() bool {
 
 // manifest lists all files present as of a specified revision.
 func (ge *GitExtractor) manifest(rev string) []manifestEntry {
-	data, err := captureFromProcess("git ls-tree -rz --full-tree " + rev)
+	data, err := captureFromProcess("git ls-tree -rz --full-tree "+rev, control.baton)
 	if err != nil {
 		panic(throw("extractor", "Couldn't spawn git ls-tree in manifest: %v", err))
 	}
@@ -605,9 +617,11 @@ func (he *HgExtractor) keepHouse() error {
 func (he *HgExtractor) capture(cmd ...string) (string, error) {
 	command := shellquote.Join(cmd...)
 	if he == nil || he.hgcl == nil {
-		return captureFromProcess(command)
+		return captureFromProcess(command, control.baton)
+	}
+	if logEnable(logCOMMANDS) {
+		logit("%s: capturing %s", rfc3339(time.Now()), command)
 	}
-	logit(logCOMMANDS, "%s: capturing %s", rfc3339(time.Now()), command)
 	stdout, stderr, err := he.hgcl.runcommand(cmd)
 	content := string(stdout) + string(stderr)
 	if logEnable(logCOMMANDS) {
@@ -619,7 +633,9 @@ func (he *HgExtractor) capture(cmd ...string) (string, error) {
 func (he *HgExtractor) mustCapture(cmd []string, errorclass string) string {
 	data, err := he.capture(cmd...)
 	if err != nil {
-		logit(logSHOUT, "%s", strings.TrimSpace(data))
+		if logEnable(logSHOUT) {
+			logit("%s", strings.TrimSpace(data))
+		}
 		panic(throw(errorclass,
 			"In %s, command %s failed: %v",
 			errorclass, shellquote.Join(cmd...), err))
@@ -640,7 +656,9 @@ func (he *HgExtractor) byLine(rs *RepoStreamer, cmd []string, errfmt string,
 	}
 	stdout, stderr, err := he.hgcl.runcommand(cmd)
 	if err != nil {
-		logit(logSHOUT, "%s", strings.TrimSpace(string(stderr)))
+		if logEnable(logSHOUT) {
+			logit("%s", strings.TrimSpace(string(stderr)))
+		}
 		return err
 	}
 	last := false
@@ -799,12 +817,13 @@ func (he *HgExtractor) gatherAllReferences(rs *RepoStreamer) error {
 		line = strings.TrimSpace(line)
 		fields := strings.Split(line, "\t")
 		h := fields[len(fields)-1]
-		n := strings.Join(fields[:len(fields)-1], "\t")
-		if n == "tip" { // pseudo-tag for most recent commit
-			return nil // We don't want it
+		for _, tag := range strings.Split(fields[0], " ") {
+			if tag == "tip" { // pseudo-tag for most recent commit
+				continue // We don't want it
+			}
+			he.tagsFound = true
+			rs.refs.set("refs/tags/"+tag, h[:12])
 		}
-		he.tagsFound = true
-		rs.refs.set("refs/tags/"+n, h[:12])
 		return nil
 	}
 	err = he.byLine(rs,
@@ -848,7 +867,9 @@ func (he *HgExtractor) gatherChildTimestamps(rs *RepoStreamer) map[string]time.T
 		// a tag will get the correct hg branch name, even if
 		// the hg branch coloring is not compatible with the
 		// git coloring algorithm
-		logit(logTOPOLOGY, "setting default branch of %s to %s", h, branch)
+		if logEnable(logTOPOLOGY) {
+			logit("setting default branch of %s to %s", h, branch)
+		}
 		rs.meta[h].branch = branch
 		// Fill in the branch tips with child timestamps to
 		// ensure that they can't be over-colored (other
@@ -866,7 +887,9 @@ func (he *HgExtractor) gatherChildTimestamps(rs *RepoStreamer) map[string]time.T
 
 func (he *HgExtractor) _branchColorItems() *OrderedMap {
 	if !he.tagsFound && !he.bookmarksFound {
-		logit(logEXTRACT, "no tags or bookmarks.")
+		if logEnable(logEXTRACT) {
+			logit("no tags or bookmarks.")
+		}
 		// If we didn't find any tags or bookmarks, we can
 		// safely color all commits using hg branch names,
 		// since hg stores them with commit metadata; note,
@@ -897,7 +920,9 @@ func (he *HgExtractor) colorBranches(rs *RepoStreamer) error {
 			if rs.meta[h] == nil {
 				rs.meta[h] = new(CommitMeta)
 			}
-			logit(logTOPOLOGY, "setting branch from color items, %s to %s", h, color)
+			if logEnable(logTOPOLOGY) {
+				logit("setting branch from color items, %s to %s", h, color)
+			}
 			rs.meta[h].branch = color
 		}
 	} else {
@@ -934,7 +959,9 @@ func (he *HgExtractor) postExtract(repo *Repository) {
 func (he *HgExtractor) isClean() bool {
 	data, err := he.capture("hg", "status", "--modified")
 	if err != nil {
-		logit(logSHOUT, "%s", strings.TrimSpace(data))
+		if logEnable(logSHOUT) {
+			logit("%s", strings.TrimSpace(data))
+		}
 		panic(throw("extractor", "Couldn't spawn hg status --modified: %v", err))
 	}
 	return data == ""
@@ -999,7 +1026,9 @@ func (he HgExtractor) manifest(rev string) []manifestEntry {
 				dest := strings.ReplaceAll(tempFile.Name(), "%", "%%")
 				data, err = he.capture("hg", "cat", "-r", rev, me.pathname, "-o", dest)
 				if err != nil {
-					logit(logSHOUT, "%s", strings.TrimSpace(data))
+					if logEnable(logSHOUT) {
+						logit("%s", strings.TrimSpace(data))
+					}
 					panic(throw("extractor", "Couldn't cat blob to hash it: %v", err))
 				}
 				hash := sha1.New()
@@ -1026,7 +1055,9 @@ func (he *HgExtractor) catFile(rev string, path string, dest string) error {
 	dest = strings.ReplaceAll(dest, "%", "%%")
 	data, err := he.capture("hg", "cat", "-r", rev, path, "-o", dest)
 	if err != nil {
-		logit(logSHOUT, "%s", strings.TrimSpace(data))
+		if logEnable(logSHOUT) {
+			logit("%s", strings.TrimSpace(data))
+		}
 	}
 	return err
 }
@@ -1035,7 +1066,9 @@ func (he *HgExtractor) catFile(rev string, path string, dest string) error {
 func (he HgExtractor) getComment(rev string) string {
 	data, err := he.capture("hg", "log", "-r", rev, "--template", `{desc}\n`)
 	if err != nil {
-		logit(logSHOUT, "%s", strings.TrimSpace(data))
+		if logEnable(logSHOUT) {
+			logit("%s", strings.TrimSpace(data))
+		}
 		panic(throw("extractor", "Couldn't spawn hg log: %v", err))
 	}
 	return data
@@ -1116,7 +1149,7 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 	}(&repo, &err)
 
 	rs.extractor.preExtract()
-	repo.makedir()
+	repo.makedir("extract")
 	front := fmt.Sprintf("#reposurgeon sourcetype %s\n", vcs.name)
 	repo.addEvent(newPassthrough(repo, front))
 
@@ -1194,8 +1227,7 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 		commit := newCommit(repo)
 		rs.baton.twirl()
 		present := rs.extractor.manifest(revision)
-		//logit(logEXTRACT,
-		//	"%s: present %v", trunc(revision), present)
+		//if logEnable(logEXTRACT) {logit("%s: present %v", trunc(revision), present)}
 		parents := rs.getParents(revision)
 		attrib, err := newAttribution(rs.getCommitter(revision))
 		if err != nil {
@@ -1216,8 +1248,7 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 		commit.Comment = rs.extractor.getComment(revision)
 		//if debugEnable(logEXTRACT) {
 		//	msg := strconv.Quote(commit.Comment)
-		//	logit(logEXTRACT,
-		//		"%s: comment '%s'", trunc(revision), msg)
+		//	logit("%s: comment '%s'", trunc(revision), msg)
 		//}
 		// Git fast-import constructs the tree from the first parent only
 		// for a merge commit; fileops from all other parents have to be
@@ -1230,8 +1261,7 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 			}
 		}
 
-		//logit(logEXTRACT,
-		//	"%s: visible files '%s'", trunc(revision), rs.visibleFiles[revision])
+		//if logEnable(logEXTRACT) {logit("%s: visible files '%s'", trunc(revision), rs.visibleFiles[revision])}
 
 		if len(present) > 0 {
 			fileList := newOrderedStringSet()
@@ -1242,14 +1272,14 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 				}
 				if mark, ok := rs.hashToMark[me.sig.hashval]; ok {
 					//if debugEnable(logEXTRACT) {
-					//	logit(logSHOUT, "%s: %s has old hash %v", trunc(revision), me.pathname, shortdump(me.sig.hashval))
+					//	logit("%s: %s has old hash %v", trunc(revision), me.pathname, shortdump(me.sig.hashval))
 					//}
 					// The file's hash corresponds
 					// to an existing blob;
 					// generate modify, copy, or
 					// rename as appropriate.
 					if _, ok := rs.visibleFiles[revision][me.pathname]; !ok || rs.visibleFiles[revision][me.pathname] != *me.sig {
-						//logit(logEXTRACT, "%s: update for %s", trunc(revision), me.pathname)
+						//if logEnable(logEXTRACT) {logit("%s: update for %s", trunc(revision), me.pathname)}
 						found := false
 						var deletia []string
 						for _, item := range deletia {
@@ -1264,14 +1294,15 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 				} else {
 					// Content hash doesn't match
 					// any existing blobs
-					//logit(logEXTRACT, "%s: %s has new hash %v",
-					//	trunc(revision), me.pathname, shortdump(me.sig.hashval))
+					//if logEnable(logEXTRACT) {
+					//	logit("%s: %s has new hash %v", trunc(revision), me.pathname, shortdump(me.sig.hashval))
+					//}
 					blobmark := markNumber(repo.newmark())
 					rs.hashToMark[me.sig.hashval] = blobmark
 					// Actual content enters the representation
 					blob := newBlob(repo)
 					blob.setMark(blobmark.String())
-					//logit(logEXTRACT, "%s: blob gets mark %s", trunc(revision), blob.mark)
+					//if logEnable(logEXTRACT) {logit("%s: blob gets mark %s", trunc(revision), blob.mark)}
 					blobfile := blob.getBlobfile(true)
 					err = rs.extractor.catFile(revision, me.pathname, blobfile)
 					if err != nil {
@@ -1308,7 +1339,7 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 		commit.properties = &newprops
 		rs.commitMap[revision] = commit
 		commit.setMark(repo.newmark())
-		//logit(logEXTRACT, "%s: commit gets mark %s (%d ops)", trunc(revision), commit.mark, len(commit.operations()))
+		//if logEnable(logEXTRACT) {logit("%s: commit gets mark %s (%d ops)", trunc(revision), commit.mark, len(commit.operations()))}
 		repo.addEvent(commit)
 		rs.baton.percentProgress(uint64(revcount))
 	}
@@ -1332,8 +1363,25 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 		}
 		return false
 	})
+	isBranchColor := func(name string) bool {
+		for rev, commit := range rs.commitMap {
+			if name == commit.Branch && rs.refs.get(name) == rev {
+				return true
+			}
+		}
+		return false
+	}
+	isAnnotatedTag := func(name string) bool {
+		for _, tag := range rs.tags {
+			if tag.tagname == name {
+				return true
+			}
+		}
+		return false
+	}
 	for _, resetname := range rs.refs.keys {
-		if !strings.Contains(resetname, "/tags/") {
+		if !isAnnotatedTag(resetname) && (!strings.Contains(resetname, "/tags/") ||
+			!isBranchColor(resetname)) {
 			committish := rs.commitMap[rs.refs.get(resetname)].mark
 			if committish == "" {
 				panic(throw("extractor", "could not get a mark for the target of %s", resetname))
@@ -1363,4 +1411,39 @@ func (rs *RepoStreamer) extract(repo *Repository, vcs *VCS) (_repo *Repository,
 	return repo, err
 }
 
+/**************************************************************************
+
+Experimental ClearCase extractor code begins here
+
+See https://stackoverflow.com/questions/60362158/in-clearcase-need-cli-invocation-to-list-all-revisions/60363075#60363075
+
+Commands used:
+
+cleartool lsbl -fmt "%n:%m:%[permissions]p:%[owner]Fp:%Nd":%l
+   %n is the full name of the element.
+   %m is the element type - typically file or directory.
+   %[permissions]p is file or directory permissions in rwx format.
+   %[owner]Fp is the user ID of the change owner.
+   %Nd will be yyyymmdd.time date; time is local in 24-hour format.
+   %l will be a comma-separated list of labels for this revision.
+   This command requires UCM.
+
+* Need one to get the parent of revision 0 on a branch.
+
+* Need one to get content from revision
+
+* Need one to get comment from revision
+
+* Need one to get filename
+
+We won't need a branch-coloring algorithm, as all revision
+names contain a branch. There are no annotated tags.
+
+How do we extract merge information?
+
+Rescue per-revision attributes?
+
+
+**************************************************************************/
+
 // end
