@@ -3,12 +3,12 @@ package main
 // The event selector. Exports one class, which is used
 // to compile and evaluate selection expressions.
 //
+// Copyright by Eric S. Raymond
 // SPDX-License-Identifier: BSD-2-Clause
 
 import (
 	"fmt"
 	"io"
-	"math"
 	"reflect"
 	"regexp"
 	"sort"
@@ -274,12 +274,9 @@ func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
 			tree = parentTree.snapshot()
 		}
 		for _, fileop := range c.operations() {
-			if fileop.op == opM && match(fileop.Path) {
+			if (fileop.op == opM || fileop.op == opC || fileop.op == opR) &&
+				match(fileop.Path) {
 				tree.set(fileop.Path, true)
-			} else if fileop.op == opC && match(fileop.Target) {
-				tree.set(fileop.Target, true)
-			} else if fileop.op == opR && match(fileop.Target) {
-				tree.set(fileop.Target, true)
 			} else if fileop.op == opD && match(fileop.Path) {
 				tree.remove(fileop.Path)
 			} else if fileop.op == opR && match(fileop.Source) {
@@ -338,6 +335,7 @@ func (rs *Reposurgeon) visibilityTypeletters() map[rune]func(int) bool {
 		'M': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.parents()) > 1 },
 		'F': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.children()) > 1 },
 		'L': func(i int) bool { c, ok := e(i).(*Commit); return ok && unclean.MatchString(c.Comment) },
+		'Q': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.getDelFlag() },
 		'I': func(i int) bool { p, ok := e(i).(decodable); return ok && !p.decodable() },
 		'D': func(i int) bool { p, ok := e(i).(alldel); return ok && p.alldeletes() },
 		'N': func(i int) bool { return rs.hasReference(e(i)) },
@@ -500,24 +498,7 @@ func (rs *Reposurgeon) evalTextSearch(state selEvalState,
 	}
 	events := rs.chosen().events
 	it := preselection.Iterator()
-	for it.Next() {
-		e := events[it.Value()]
-		if checkBranch {
-			if t, ok := e.(*Tag); ok {
-				e = rs.repo.markToEvent(t.committish)
-			} else if b, ok := e.(*Blob); ok {
-				for ci := it.Value(); ci < len(events); ci++ {
-					possible := events[ci]
-					if c, ok := possible.(*Commit); ok &&
-						c.references(b.mark) {
-						// FIXME: Won't find multiple
-						// references
-						e = possible
-						break
-					}
-				}
-			}
-		}
+	conditionalMark := func(e Event) {
 		for _, searchable := range searchIn {
 			if _, ok := getAttr(e, searchable); ok {
 				key := extractors[searchable](e)
@@ -543,6 +524,23 @@ func (rs *Reposurgeon) evalTextSearch(state selEvalState,
 			}
 		}
 	}
+	for it.Next() {
+		e := events[it.Value()]
+		if checkBranch {
+			if t, ok := e.(*Tag); ok {
+				e = rs.repo.markToEvent(t.committish)
+			} else if b, ok := e.(*Blob); ok {
+				for ci := it.Value(); ci < len(events); ci++ {
+					possible := events[ci]
+					if c, ok := possible.(*Commit); ok &&
+						c.references(b.mark) {
+						conditionalMark(possible)
+					}
+				}
+			}
+		}
+		conditionalMark(e)
+	}
 	return matchers
 }
 
@@ -828,6 +826,9 @@ func (p *SelectionParser) parseTerm() selEvaluator {
 
 func (p *SelectionParser) evalTermNegate(state selEvalState,
 	preselection *fastOrderedIntSet, op selEvaluator) *fastOrderedIntSet {
+	if op == nil {
+		panic(throw("command", "inner expression to be negated is invalid"))
+	}
 	negated := op(state, state.allItems())
 	remainder := newFastOrderedIntSet()
 	for i, n := 0, state.nItems(); i < n; i++ {
@@ -932,8 +933,8 @@ func (p *SelectionParser) parsePolyrange() selEvaluator {
 	return polyrange
 }
 
-const polyrangeRange = math.MinInt64
-const polyrangeDollar = math.MaxInt64
+const polyrangeRange = minInt
+const polyrangeDollar = maxInt
 
 // evalPolyrange evaluates a polyrange specification (list of intervals)
 func (p *SelectionParser) evalPolyrange(state selEvalState,
@@ -949,7 +950,7 @@ func (p *SelectionParser) evalPolyrange(state selEvalState,
 	}
 	// Resolve spans
 	resolved := newFastOrderedIntSet()
-	last := int(math.MinInt64)
+	last := minInt
 	spanning := false
 	it := selection.Iterator()
 	for it.Next() {
@@ -958,7 +959,7 @@ func (p *SelectionParser) evalPolyrange(state selEvalState,
 			i = state.nItems() - 1
 		}
 		if i == polyrangeRange { // ".."
-			if last == math.MinInt64 {
+			if last == minInt {
 				panic(throw("command", "start of span is missing"))
 			}
 			spanning = true
