@@ -9,14 +9,9 @@ echo Return code: $? >>/tmp/out$$
 cd - >/dev/null || ( echo "$0: cd failed"; exit 1 )
 ./dir-md5 /tmp/test-workdir$$ >>/tmp/out$$
 
-case $1 in
-    --regress)
-        diff --text -u repotool-initialize-cvs-git.chk /tmp/out$$ || ( echo "$0: FAILED"; exit 1 ); ;;
-    --rebuild)
-	cat /tmp/out$$ >repotool-initialize-cvs-git.chk;;
-    --view)
-	cat /tmp/out$$;;
-esac
+# shellcheck disable=SC1091
+. ./common-setup.sh
+toolmeta "$1" /tmp/out$$
 
 st=$?
 if [ $st -eq 0 ]; then
