@@ -4,14 +4,14 @@
 # This is how we detect we're in Gitlab CI.
 if [ -z "${USER}" ]
 then
-    echo "SKIPPED - ssh is blocked in CI, so rsync will fail"
+    echo "not ok - ssh is blocked in CI, so rsync will fail # SKIP"
     exit 0
 fi
 
-command -v cvs >/dev/null 2>&1 || { echo "    Skipped, cvs missing."; exit 0; }
-command -v cvs-fast-export >/dev/null 2>&1 || { echo "    Skipped, cvs-fast-export missing."; exit 0; }
+# shellcheck disable=SC1091
+. ./common-setup.sh
 
-set -e
+need cvs cvs-fast-export
 
 TMPDIR=${TMPDIR:-/tmp}
 
@@ -20,7 +20,7 @@ trap 'rm -rf ${TMPDIR}/cvs-scratch$$ ${TMPDIR}/ref$$ ${TMPDIR}/out$$' EXIT HUP I
 # Go to our sandbox
 here=$(realpath .)
 mkdir "${TMPDIR}/cvs-scratch$$"
-cd "${TMPDIR}/cvs-scratch$$" >/dev/null || (echo "$0: cd failed" >&2; exit 1)
+tapcd "${TMPDIR}/cvs-scratch$$"
 
 # Make the workflow file.
 repotool initialize -q hack1 cvs git
@@ -29,7 +29,9 @@ repotool initialize -q hack1 cvs git
 make --silent -e REMOTE_URL="cvs://localhost${here}/hack1.repo#module" VERBOSITY="" 2>&1 | sed "/ no commitids before/"d
 
 # Compare the results
-repotool compare-all hack1-mirror hack1-git || echo "FAILED: Repositories do not compare equal."
+repotool compare-all hack1-mirror hack1-git || ( echo "not ok - $0: repositories do not compare equal."; exit 1 )
+
+echo "ok - $0: repositories compare equal"
 
 # No output is good news
 
